/*
 * Copyright 2024 KylinSoft Co., Ltd.
 *
 * This program is free software: you can redistribute it and/or modify it under
 * the terms of the GNU General Public License as published by the Free Software
 * Foundation, either version 3 of the License, or (at your option) any later
 * version.
 *
 * This program is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
 * FOR A PARTICULAR PURPOSE. See the GNU General Public License for more
 * details.
 *
 * You should have received a copy of the GNU General Public License along with
 * this program. If not, see <https://www.gnu.org/licenses/>.
 */

#ifndef SERVICES_IMAGEEMBEDDINGPROCESSORPROXY_H
#define SERVICES_IMAGEEMBEDDINGPROCESSORPROXY_H

#include <gio/gio.h>

#include <any>
#include <functional>
#include <string>
#include <vector>

#include "coreimageembeddingserviceglue.h"
#include "embeddingcommon.h"

class ImageEmbeddingProcessorProxy {
public:
    using ImageEmbeddingCallback = std::function<void(EmbeddingResult* result)>;
    using ImageEmbeddingResultCallback =
        std::function<void(EmbeddingResult* result, void* user_data)>;
    struct EmbeddingRequestContext {
        ImageEmbeddingResultCallback callback;
        void* callback_user_data;
    };

public:
    ~ImageEmbeddingProcessorProxy();

    int connect();
    int initEngine();

    EmbeddingResult* embeddingText(const std::string& text);

    EmbeddingResult* embeddingImage(const std::string& file_path);

    EmbeddingResult* embeddingBase64Image(const std::string& file_data);

    void embeddingTextAsync(const std::string& text,
                            ImageEmbeddingResultCallback callback,
                            void* callback_user_data);

    void embeddingImageAsync(const std::string& file_path,
                             ImageEmbeddingResultCallback callback,
                             void* callback_user_data);

    void embeddingBase64ImageAsync(const std::string& file_data,
                                   ImageEmbeddingResultCallback callback,
                                   void* callback_user_data);
    std::string getModelInfo();

private:
    void destroy();

    static EmbeddingResult parseResult(const std::string& json_string_result);

    static void embeddingTextAsyncCallback(GObject* object, GAsyncResult* res,
                                           gpointer data);
    static void embeddingImageAsyncCallback(GObject* object, GAsyncResult* res,
                                            gpointer data);
    static void embeddingBase64ImageAsyncCallback(GObject* object,
                                                  GAsyncResult* res,
                                                  gpointer data);

private:
    AiRuntimeCoreImageEmbeddingService* delegate_ = nullptr;
    GDBusConnection* connection_ = nullptr;

    int sessionId_ = -1;
    int initErrorCode_ = -1;  // 0表示初始化成功，非0表示初始化失败
};

#endif